@@ -2,13 +2,13 @@ import os
 import re
 from difflib import SequenceMatcher
 from itertools import chain
+from os.path import join as pjoin
 
 from lxml import etree
 from pkgcore import const as pkgcore_const
 from pkgcore.ebuild.atom import MalformedAtom, atom
-from pkgcore.restrictions.packages import Conditional
 from pkgcore.fetch import fetchable
-from snakeoil.osutils import pjoin
+from pkgcore.restrictions.packages import Conditional
 from snakeoil.sequences import iflatten_instance
 from snakeoil.strings import pluralism
 
@@ -25,10 +25,10 @@ class _MissingXml(results.Error):
 
     @property
     def desc(self):
-        return f'{self._attr} is missing {self.filename}'
+        return f"{self._attr} is missing {self.filename}"
 
 
-class _BadlyFormedXml(results.Warning):
+class _BadlyFormedXml(results.Error):
     """XML isn't well formed."""
 
     def __init__(self, filename, error, **kwargs):
@@ -38,7 +38,7 @@ class _BadlyFormedXml(results.Warning):
 
     @property
     def desc(self):
-        return f'{self._attr} {self.filename} is not well formed xml: {self.error}'
+        return f"{self._attr} {self.filename} is not well formed xml: {self.error}"
 
 
 class _InvalidXml(results.Error):
@@ -51,7 +51,7 @@ class _InvalidXml(results.Error):
 
     @property
     def desc(self):
-        return f'{self._attr} {self.filename} violates metadata.xsd:\n{self.message}'
+        return f"{self._attr} {self.filename} violates metadata.xsd:\n{self.message}"
 
 
 class _MetadataXmlInvalidPkgRef(results.Error):
@@ -65,8 +65,8 @@ class _MetadataXmlInvalidPkgRef(results.Error):
     @property
     def desc(self):
         return (
-            f'{self._attr} {self.filename} <pkg/> '
-            f'references unknown/invalid package: {self.pkgtext!r}'
+            f"{self._attr} {self.filename} <pkg/> "
+            f"references unknown/invalid package: {self.pkgtext!r}"
         )
 
 
@@ -81,8 +81,8 @@ class _MetadataXmlInvalidCatRef(results.Error):
     @property
     def desc(self):
         return (
-            f'{self._attr} {self.filename} <cat/> references '
-            f'unknown/invalid category: {self.cattext!r}'
+            f"{self._attr} {self.filename} <cat/> references "
+            f"unknown/invalid category: {self.cattext!r}"
         )
 
 
@@ -97,8 +97,8 @@ class MaintainerNeeded(results.PackageResult, results.Warning):
     @property
     def desc(self):
         if not self.needed:
-            return f'{self.filename}: missing maintainer-needed comment'
-        return f'{self.filename}: invalid maintainer-needed comment'
+            return f"{self.filename}: missing maintainer-needed comment"
+        return f"{self.filename}: invalid maintainer-needed comment"
 
 
 class MaintainerWithoutProxy(results.PackageResult, results.Warning):
@@ -119,8 +119,8 @@ class MaintainerWithoutProxy(results.PackageResult, results.Warning):
     @property
     def desc(self):
         s = pluralism(self.maintainers)
-        maintainers = ', '.join(self.maintainers)
-        return f'{self.filename}: proxied maintainer{s} missing proxy dev/project: {maintainers}'
+        maintainers = ", ".join(self.maintainers)
+        return f"{self.filename}: proxied maintainer{s} missing proxy dev/project: {maintainers}"
 
 
 class ProxyWithoutProxied(results.PackageResult, results.Warning):
@@ -137,7 +137,7 @@ class ProxyWithoutProxied(results.PackageResult, results.Warning):
 
     @property
     def desc(self):
-        return f'{self.filename}: proxy with no proxied maintainer'
+        return f"{self.filename}: proxy with no proxied maintainer"
 
 
 class NonexistentProjectMaintainer(results.PackageResult, results.Warning):
@@ -151,8 +151,8 @@ class NonexistentProjectMaintainer(results.PackageResult, results.Warning):
     @property
     def desc(self):
         s = pluralism(self.emails)
-        emails = ', '.join(self.emails)
-        return f'{self.filename}: nonexistent project maintainer{s}: {emails}'
+        emails = ", ".join(self.emails)
+        return f"{self.filename}: nonexistent project maintainer{s}: {emails}"
 
 
 class WrongMaintainerType(results.PackageResult, results.Warning):
@@ -166,7 +166,7 @@ class WrongMaintainerType(results.PackageResult, results.Warning):
     @property
     def desc(self):
         s = pluralism(self.emails)
-        emails = ', '.join(self.emails)
+        emails = ", ".join(self.emails)
         return f'{self.filename}: project maintainer{s} with type="person": {emails}'
 
 
@@ -222,7 +222,7 @@ class _MetadataXmlIndentation(results.BaseLinesResult, results.Style):
 
     @property
     def desc(self):
-        return f'{self.filename}: metadata.xml has inconsistent indentation {self.lines_str}'
+        return f"{self.filename}: metadata.xml has inconsistent indentation {self.lines_str}"
 
 
 class CatMetadataXmlIndentation(_MetadataXmlIndentation, results.CategoryResult):
@@ -250,7 +250,7 @@ class _MetadataXmlEmptyElement(results.Style):
 
     @property
     def desc(self):
-        return f'{self.filename}: empty element {self.element!r} on line {self.line}'
+        return f"{self.filename}: empty element {self.element!r} on line {self.line}"
 
 
 class CatMetadataXmlEmptyElement(_MetadataXmlEmptyElement, results.CategoryResult):
@@ -288,8 +288,23 @@ class InvalidRemoteID(results.PackageResult, results.Warning):
 
     @property
     def desc(self):
-        return (f"remote-id value {self.id_value!r} invalid for "
-                f"type={self.id_type!r}, expected: {self.expected!r}")
+        return (
+            f"remote-id value {self.id_value!r} invalid for "
+            f"type={self.id_type!r}, expected: {self.expected!r}"
+        )
+
+
+class InvalidMetadataRestrict(results.PackageResult, results.Error):
+    """Invalid package restrictions used in metadata.xml."""
+
+    def __init__(self, restrict: str, msg: str, **kwargs):
+        super().__init__(**kwargs)
+        self.restrict = restrict
+        self.msg = msg
+
+    @property
+    def desc(self):
+        return f"metadata.xml: invalid package restrictions {self.restrict!r}: {self.msg}"
 
 
 class _XmlBaseCheck(Check):
@@ -306,13 +321,12 @@ class _XmlBaseCheck(Check):
         self.repo_base = self.options.target_repo.location
         self.pkgref_cache = {}
         # content validation checks to run after parsing XML doc
-        self._checks = tuple(
-            getattr(self, x) for x in dir(self) if x.startswith('_check_'))
+        self._checks = tuple(getattr(self, x) for x in dir(self) if x.startswith("_check_"))
 
         # Prefer xsd file from the target repository or its masters, falling
         # back to the file installed with pkgcore.
         for repo in reversed(self.options.target_repo.trees):
-            metadata_xsd = pjoin(repo.location, 'metadata', 'xml-schema', 'metadata.xsd')
+            metadata_xsd = pjoin(repo.location, "metadata", "xml-schema", "metadata.xsd")
             if os.path.isfile(metadata_xsd):
                 try:
                     self.schema = etree.XMLSchema(etree.parse(metadata_xsd))
@@ -321,7 +335,7 @@ class _XmlBaseCheck(Check):
                     # ignore invalid xsd files
                     pass
         else:
-            metadata_xsd = pjoin(pkgcore_const.DATA_PATH, 'xml-schema', 'metadata.xsd')
+            metadata_xsd = pjoin(pkgcore_const.DATA_PATH, "xml-schema", "metadata.xsd")
             self.schema = etree.XMLSchema(etree.parse(metadata_xsd))
 
     def _check_doc(self, pkg, loc, doc):
@@ -330,16 +344,19 @@ class _XmlBaseCheck(Check):
         # 'stabilize-allarches' which is allowed to be empty and 'flag' which
         # is caught by MissingLocalUseDesc.
         for el in doc.getroot().iterdescendants():
-            if (not el.getchildren() and (el.text is None or not el.text.strip())
-                    and el.tag not in ('flag', 'stabilize-allarches')):
+            if (
+                not el.getchildren()
+                and (el.text is None or not el.text.strip())
+                and el.tag not in ("flag", "stabilize-allarches")
+            ):
                 yield self.empty_element(os.path.basename(loc), el.tag, el.sourceline, pkg=pkg)
 
-        for el in doc.findall('.//cat'):
+        for el in doc.findall(".//cat"):
             c = el.text.strip()
             if c not in self.options.search_repo.categories:
                 yield self.catref_error(os.path.basename(loc), c, pkg=pkg)
 
-        for el in doc.findall('.//pkg'):
+        for el in doc.findall(".//pkg"):
             p = el.text.strip()
             if p not in self.pkgref_cache:
                 try:
@@ -358,7 +375,7 @@ class _XmlBaseCheck(Check):
         indents = set()
         with open(loc) as f:
             for lineno, line in enumerate(f, 1):
-                for i in line[:-len(line.lstrip())]:
+                for i in line[: -len(line.lstrip())]:
                     if i != orig_indent:
                         if orig_indent is None:
                             orig_indent = i
@@ -370,7 +387,7 @@ class _XmlBaseCheck(Check):
     @staticmethod
     def _format_lxml_errors(error_log):
         for x in error_log:
-            yield f'line {x.line}, col {x.column}: ({x.type_name}) {x.message}'
+            yield f"line {x.line}, col {x.column}: ({x.type_name}) {x.message}"
 
     def _parse_xml(self, pkg, loc):
         try:
@@ -387,7 +404,7 @@ class _XmlBaseCheck(Check):
         # note: while doc is available, do not pass it here as it may
         # trigger undefined behavior due to incorrect structure
         if self.schema is not None and not self.schema.validate(doc):
-            message = '\n'.join(self._format_lxml_errors(self.schema.error_log))
+            message = "\n".join(self._format_lxml_errors(self.schema.error_log))
             yield self.invalid_error(os.path.basename(loc), message, pkg=pkg)
             return
 
@@ -413,64 +430,76 @@ class PackageMetadataXmlCheck(_XmlBaseCheck):
     indent_error = PkgMetadataXmlIndentation
     empty_element = PkgMetadataXmlEmptyElement
 
-    known_results = frozenset([
-        PkgBadlyFormedXml, PkgInvalidXml, PkgMissingMetadataXml,
-        PkgMetadataXmlInvalidPkgRef, PkgMetadataXmlInvalidCatRef,
-        PkgMetadataXmlIndentation, PkgMetadataXmlEmptyElement, MaintainerNeeded,
-        MaintainerWithoutProxy, ProxyWithoutProxied, RedundantLongDescription,
-        NonexistentProjectMaintainer, WrongMaintainerType, InvalidRemoteID,
-    ])
+    known_results = frozenset(
+        [
+            PkgBadlyFormedXml,
+            PkgInvalidXml,
+            PkgMissingMetadataXml,
+            PkgMetadataXmlInvalidPkgRef,
+            PkgMetadataXmlInvalidCatRef,
+            PkgMetadataXmlIndentation,
+            PkgMetadataXmlEmptyElement,
+            MaintainerNeeded,
+            MaintainerWithoutProxy,
+            ProxyWithoutProxied,
+            RedundantLongDescription,
+            NonexistentProjectMaintainer,
+            WrongMaintainerType,
+            InvalidRemoteID,
+            InvalidMetadataRestrict,
+        ]
+    )
 
-    _one_component_validator_re = re.compile(r'^[^/]+$')
-    _two_components_validator_re = re.compile(r'^[^/]+/[^/]+$')
-    _gitlab_validator_re = re.compile(r'^([^/]+/)*[^/]+/[^/]+$')
+    _one_component_validator_re = re.compile(r"^[^/]+$")
+    _two_components_validator_re = re.compile(r"^[^/]+/[^/]+$")
+    _gitlab_validator_re = re.compile(r"^([^/]+/)*[^/]+/[^/]+$")
 
     remote_id_validators = {
         # {name}-style remotes
-        'cpan': (_one_component_validator_re, '{project}'),
-        'cpan-module': (_one_component_validator_re, '{module}'),
-        'cran': (_one_component_validator_re, '{project}'),
-        'ctan': (_one_component_validator_re, '{project}'),
-        'google-code': (_one_component_validator_re, '{project}'),
-        'osdn': (_one_component_validator_re, '{project}'),
-        'pear': (_one_component_validator_re, '{project}'),
-        'pecl': (_one_component_validator_re, '{project}'),
-        'pypi': (_one_component_validator_re, '{project}'),
-        'rubygems': (_one_component_validator_re, '{project}'),
-        'sourceforge': (_one_component_validator_re, '{project}'),
+        "cpan": (_one_component_validator_re, "{project}"),
+        "cpan-module": (_one_component_validator_re, "{module}"),
+        "cran": (_one_component_validator_re, "{project}"),
+        "ctan": (_one_component_validator_re, "{project}"),
+        "google-code": (_one_component_validator_re, "{project}"),
+        "osdn": (_one_component_validator_re, "{project}"),
+        "pear": (_one_component_validator_re, "{project}"),
+        "pecl": (_one_component_validator_re, "{project}"),
+        "pypi": (_one_component_validator_re, "{project}"),
+        "rubygems": (_one_component_validator_re, "{project}"),
+        "sourceforge": (_one_component_validator_re, "{project}"),
         # {name} with a special check for lp: prefix
-        'launchpad': (re.compile(r'^(?!lp:)[^/]+$'), '{project}'),
+        "launchpad": (re.compile(r"^(?!lp:)[^/]+$"), "{project}"),
         # {owner}/{name}-style remotes
-        'bitbucket': (_two_components_validator_re, '{username}/{project}'),
-        'github': (_two_components_validator_re, '{username}/{project}'),
+        "bitbucket": (_two_components_validator_re, "{username}/{project}"),
+        "codeberg": (_two_components_validator_re, "{username}/{project}"),
+        "github": (_two_components_validator_re, "{username}/{project}"),
         # gitlab (2+ components)
-        'gitlab': (_gitlab_validator_re, '{username}/[{group}/...]{repo}'),
-        'heptapod': (_gitlab_validator_re, '{username}/[{group}/...]{repo}'),
+        "gitlab": (_gitlab_validator_re, "{username}/[{group}/...]{repo}"),
+        "heptapod": (_gitlab_validator_re, "{username}/[{group}/...]{repo}"),
         # cpe
-        'cpe': (re.compile(r'^cpe:/[aho]:[^:]+:[^:]+$'),
-                'cpe:/[aho]:{vendor}:{product}'),
+        "cpe": (re.compile(r"^cpe:/[aho]:[^:]+:[^:]+$"), "cpe:/[aho]:{vendor}:{product}"),
         # 1+ component + no ".git" suffix
-        'gentoo': (re.compile(r'^([^/]+/)*[^/]+(?<!\.git)$'),
-                   '[{group}/...]{repo}'),
+        "gentoo": (re.compile(r"^([^/]+/)*[^/]+(?<!\.git)$"), "[{group}/...]{repo}"),
         # a positive decimal number
-        'vim': (re.compile(r'^[1-9]\d*$'), '{script_id}'),
+        "vim": (re.compile(r"^[1-9]\d*$"), "{script_id}"),
     }
 
     @staticmethod
     def _maintainer_proxied_key(m):
         if m.proxied is not None:
             return m.proxied
-        if m.email == 'proxy-maint@gentoo.org':
-            return 'proxy'
-        if m.email.endswith('@gentoo.org'):
-            return 'no'
-        return 'yes'
+        if m.email == "proxy-maint@gentoo.org":
+            return "proxy"
+        if m.email.endswith("@gentoo.org"):
+            return "no"
+        return "yes"
 
     def _check_maintainers(self, pkg, loc, doc):
         """Validate maintainers in package metadata for the gentoo repo."""
         if self.options.gentoo_repo:
             maintainer_needed = any(
-                c.text.strip() == 'maintainer-needed' for c in doc.xpath('//comment()'))
+                c.text.strip() == "maintainer-needed" for c in doc.xpath("//comment()")
+            )
             if pkg.maintainers:
                 # check for invalid maintainer-needed comment
                 if maintainer_needed:
@@ -478,15 +507,14 @@ class PackageMetadataXmlCheck(_XmlBaseCheck):
 
                 # determine proxy maintainer status
                 proxied, devs, proxies = [], [], []
-                proxy_map = {'yes': proxied, 'no': devs, 'proxy': proxies}
+                proxy_map = {"yes": proxied, "no": devs, "proxy": proxies}
                 for m in pkg.maintainers:
                     proxy_map[self._maintainer_proxied_key(m)].append(m)
 
                 # check proxy maintainers
                 if not devs and not proxies:
                     maintainers = sorted(map(str, pkg.maintainers))
-                    yield MaintainerWithoutProxy(
-                        os.path.basename(loc), maintainers, pkg=pkg)
+                    yield MaintainerWithoutProxy(os.path.basename(loc), maintainers, pkg=pkg)
                 elif not proxied and proxies:
                     yield ProxyWithoutProxied(os.path.basename(loc), pkg=pkg)
             elif not maintainer_needed:
@@ -498,27 +526,49 @@ class PackageMetadataXmlCheck(_XmlBaseCheck):
                 nonexistent = []
                 wrong_maintainers = []
                 for m in pkg.maintainers:
-                    if m.maint_type == 'project' and m.email not in projects:
+                    if m.maint_type == "project" and m.email not in projects:
                         nonexistent.append(m.email)
-                    elif m.maint_type == 'person' and m.email in projects:
+                    elif m.maint_type == "person" and m.email in projects:
                         wrong_maintainers.append(m.email)
                 if nonexistent:
                     yield NonexistentProjectMaintainer(
-                        os.path.basename(loc), sorted(nonexistent), pkg=pkg)
+                        os.path.basename(loc), sorted(nonexistent), pkg=pkg
+                    )
                 if wrong_maintainers:
                     yield WrongMaintainerType(
-                        os.path.basename(loc), sorted(wrong_maintainers), pkg=pkg)
+                        os.path.basename(loc), sorted(wrong_maintainers), pkg=pkg
+                    )
 
     def _check_longdescription(self, pkg, loc, doc):
         if pkg.longdescription is not None:
             match_ratio = SequenceMatcher(None, pkg.description, pkg.longdescription).ratio()
             if match_ratio > 0.75:
-                msg = 'metadata.xml longdescription closely matches DESCRIPTION'
+                msg = "metadata.xml longdescription closely matches DESCRIPTION"
                 yield RedundantLongDescription(msg, pkg=pkg)
-            elif len(pkg.longdescription) < 100:
-                msg = 'metadata.xml longdescription is too short'
+            elif len(pkg.longdescription) < 80:
+                msg = "metadata.xml longdescription is too short"
                 yield RedundantLongDescription(msg, pkg=pkg)
 
+    def _check_restricts(self, pkg, loc, doc):
+        restricts = (
+            c.get("restrict")
+            for path in ("maintainer", "use/flag")
+            for c in doc.xpath(f"/pkgmetadata/{path}[string(@restrict)]")
+        )
+        for restrict_str in restricts:
+            try:
+                restrict = atom(restrict_str, eapi="0")
+                if restrict.key != pkg.key:
+                    yield InvalidMetadataRestrict(
+                        restrict_str, "references another package", pkg=pkg
+                    )
+                if restrict.use:
+                    yield InvalidMetadataRestrict(
+                        restrict_str, "USE-conditionals are prohibited", pkg=pkg
+                    )
+            except MalformedAtom as exc:
+                yield InvalidMetadataRestrict(restrict_str, exc, pkg=pkg)
+
     def _check_remote_id(self, pkg, loc, doc):
         for u in pkg.upstreams:
             # empty values are already reported as PkgMetadataXmlEmptyElement
@@ -533,13 +583,13 @@ class PackageMetadataXmlCheck(_XmlBaseCheck):
 
     def _get_xml_location(self, pkg):
         """Return the metadata.xml location for a given package."""
-        return pjoin(os.path.dirname(pkg.ebuild.path), 'metadata.xml')
+        return pjoin(os.path.dirname(pkg.ebuild.path), "metadata.xml")
 
 
 class CategoryMetadataXmlCheck(_XmlBaseCheck):
     """Category level metadata.xml scans."""
 
-    _source = (sources.CategoryRepoSource, (), (('source', sources.RawRepoSource),))
+    _source = (sources.CategoryRepoSource, (), (("source", sources.RawRepoSource),))
     misformed_error = CatBadlyFormedXml
     invalid_error = CatInvalidXml
     missing_error = CatMissingMetadataXml
@@ -548,15 +598,21 @@ class CategoryMetadataXmlCheck(_XmlBaseCheck):
     indent_error = CatMetadataXmlIndentation
     empty_element = CatMetadataXmlEmptyElement
 
-    known_results = frozenset([
-        CatBadlyFormedXml, CatInvalidXml, CatMissingMetadataXml,
-        CatMetadataXmlInvalidPkgRef, CatMetadataXmlInvalidCatRef,
-        CatMetadataXmlIndentation, CatMetadataXmlEmptyElement,
-    ])
+    known_results = frozenset(
+        [
+            CatBadlyFormedXml,
+            CatInvalidXml,
+            CatMissingMetadataXml,
+            CatMetadataXmlInvalidPkgRef,
+            CatMetadataXmlInvalidCatRef,
+            CatMetadataXmlIndentation,
+            CatMetadataXmlEmptyElement,
+        ]
+    )
 
     def _get_xml_location(self, pkg):
         """Return the metadata.xml location for a given package's category."""
-        return pjoin(self.repo_base, pkg.category, 'metadata.xml')
+        return pjoin(self.repo_base, pkg.category, "metadata.xml")
 
 
 class MissingRemoteId(results.PackageResult, results.Info):
@@ -577,8 +633,10 @@ class MissingRemoteId(results.PackageResult, results.Info):
 
     @property
     def desc(self):
-        return (f'missing remote-id of type {self.remote_type!r} with '
-            f'value {self.value!r} (inferred from URI {self.uri!r})')
+        return (
+            f'missing <remote-id type="{self.remote_type}">'
+            f"{self.value}</remote-id> (inferred from URI {self.uri!r})"
+        )
 
 
 class MissingRemoteIdCheck(Check):
@@ -587,37 +645,61 @@ class MissingRemoteIdCheck(Check):
     _source = sources.PackageRepoSource
     known_results = frozenset([MissingRemoteId])
 
-    _gitlab_match = r'(?P<value>(\w[^/]*/)*\w[^/]*/\w[^/]*)'
+    # Exclude api groups and raw project names to conform with https://docs.gitlab.com/ee/user/reserved_names.html
+    # with the URI's which are most likely to end up in SRC_URI
+    _gitlab_match = r"(?P<value>((?!api/)\w[^/]*/)+(?!raw/)\w[^/]*)"
 
     remotes_map = (
-        ('bitbucket', r'https://bitbucket.org/(?P<value>[^/]+/[^/]+)'),
-        ('freedesktop-gitlab', rf'https://gitlab.freedesktop.org/{_gitlab_match}'),
-        ('github', r'https://github.com/(?P<value>[^/]+/[^/]+)'),
-        ('gitlab', rf'https://gitlab.com/{_gitlab_match}'),
-        ('gnome-gitlab', rf'https://gitlab.gnome.org/{_gitlab_match}'),
-        ('heptapod', rf'https://foss.heptapod.net/{_gitlab_match}'),
-        ('launchpad', r'https://launchpad.net/(?P<value>[^/]+)'),
-        ('pypi', r'https://pypi.org/project/(?P<value>[^/]+)'),
-        ('pypi', r'https://files.pythonhosted.org/packages/source/\S/(?P<value>[^/]+)'),
-        ('savannah', r'https://savannah.gnu.org/projects/(?P<value>[^/]+)'),
-        ('savannah-nongnu', r'https://savannah.nongnu.org/projects/(?P<value>[^/]+)'),
-        ('sourceforge', r'https://downloads.sourceforge.(net|io)/(?:project/)?(?P<value>[^/]+)'),
-        ('sourceforge', r'https://sourceforge.(net|io)/projects/(?P<value>[^/]+)'),
-        ('sourceforge', r'https://(?P<value>[^/]+).sourceforge.(net|io)/'),
-        ('sourcehut', r'https://sr.ht/(?P<value>[^/]+/[^/]+)'),
+        ("bitbucket", r"https://bitbucket.org/(?P<value>[^/]+/[^/]+)"),
+        ("codeberg", r"https://codeberg.org/(?P<value>[^/]+/[^/]+)"),
+        ("freedesktop-gitlab", rf"https://gitlab.freedesktop.org/{_gitlab_match}"),
+        ("github", r"https://github.com/(?P<value>[^/]+/[^/]+)"),
+        ("gitlab", rf"https://gitlab.com/{_gitlab_match}"),
+        ("gnome-gitlab", rf"https://gitlab.gnome.org/{_gitlab_match}"),
+        ("heptapod", rf"https://foss.heptapod.net/{_gitlab_match}"),
+        ("kde-invent", rf"https://invent.kde.org/{_gitlab_match}"),
+        ("launchpad", r"https://launchpad.net/(?P<value>[^/]+)"),
+        ("pypi", r"https://pypi.org/project/(?P<value>[^/]+)"),
+        ("pypi", r"https://files.pythonhosted.org/packages/source/\S/(?P<value>[^/]+)"),
+        ("savannah", r"https://savannah.gnu.org/projects/(?P<value>[^/]+)"),
+        ("savannah-nongnu", r"https://savannah.nongnu.org/projects/(?P<value>[^/]+)"),
+        ("sourceforge", r"https://downloads.sourceforge.(net|io)/(?:project/)?(?P<value>[^/]+)"),
+        ("sourceforge", r"https://sourceforge.(net|io)/projects/(?P<value>[^/]+)"),
+        ("sourceforge", r"https://(?P<value>[^/]+).sourceforge.(net|io)/"),
+        ("sourcehut", r"https://sr.ht/(?P<value>[^/]+/[^/]+)"),
     )
 
     def __init__(self, options, **kwargs):
         super().__init__(options, **kwargs)
-        self.remotes_map = tuple((remote_type, re.compile(regex)) for remote_type, regex in self.remotes_map)
+        self.remotes_map = tuple(
+            (remote_type, re.compile(regex)) for remote_type, regex in self.remotes_map
+        )
+
+    @staticmethod
+    def __filter_url(url: str) -> bool:
+        if url.endswith((".patch", ".diff")):
+            return False
+        if "-crates.tar" in url:
+            return False
+        if "-deps.tar" in url:
+            return False
+        return True
 
     def feed(self, pkgset):
         remotes = {u.type: (None, None) for u in pkgset[0].upstreams}
         for pkg in sorted(pkgset, reverse=True):
-            fetchables = iflatten_instance(pkg.generate_fetchables(allow_missing_checksums=True,
-                ignore_unknown_mirrors=True, skip_default_mirrors=True), (fetchable, Conditional))
-            all_urls = set(chain.from_iterable(f.uri for f in fetchables if isinstance(f, fetchable)))
-            urls = {url for url in all_urls if not url.endswith(('.patch', '.diff'))}
+            fetchables = iflatten_instance(
+                pkg.generate_fetchables(
+                    allow_missing_checksums=True,
+                    ignore_unknown_mirrors=True,
+                    skip_default_mirrors=True,
+                ),
+                (fetchable, Conditional),
+            )
+            all_urls: set[str] = set(
+                chain.from_iterable(f.uri for f in fetchables if isinstance(f, fetchable))
+            )
+            urls = set(filter(self.__filter_url, all_urls))
             urls = sorted(urls.union(pkg.homepage), key=len)
 
             for remote_type, regex in self.remotes_map:
@@ -625,7 +707,7 @@ class MissingRemoteIdCheck(Check):
                     continue
                 for url in urls:
                     if mo := regex.match(url):
-                        remotes[remote_type] = (mo.group('value'), url)
+                        remotes[remote_type] = (mo.group("value"), url)
                         break
 
         for remote_type, (value, url) in remotes.items():
