@@ -1,11 +1,12 @@
 import os
+from os.path import join as pjoin
 from unittest.mock import patch
 
 import pytest
+from pkgcore.restrictions import packages
+
 from pkgcheck import addons
 from pkgcheck.base import PkgcheckUserException
-from pkgcore.restrictions import packages
-from snakeoil.osutils import pjoin
 
 from ..misc import FakePkg, FakeProfile, Profile
 
