@@ -1,15 +1,16 @@
 import os
 import textwrap
+from os.path import join as pjoin
 from time import sleep
 from unittest.mock import patch
 
 import pytest
+from snakeoil.fileutils import touch
+
 from pkgcheck.addons import init_addon
+from pkgcheck.addons.caches import CacheDisabled
 from pkgcheck.addons.eclass import Eclass, EclassAddon
 from pkgcheck.base import PkgcheckUserException
-from pkgcheck.addons.caches import CacheDisabled
-from snakeoil.fileutils import touch
-from snakeoil.osutils import pjoin
 
 
 class TestEclass:
