@@ -1,6 +1,8 @@
+from os.path import join as pjoin
+
 import pytest
+
 from pkgcheck import addons, feeds
-from snakeoil.osutils import pjoin
 
 from .misc import FakePkg, Profile
 
